unit main_form;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, ActnList, StdCtrls, Spin;

Const  // Konstante koje ukazuju na pravac kretanja loptice
       // Koristimo konstante za tastaturu da skratimo vreme
       // potrebno za obradu informacija...
       PRAVAC_LEVO = VK_LEFT;
       PRAVAC_DESNO = VK_RIGHT;
       PRAVAC_GORE = VK_UP;
       PRAVAC_DOLE = VK_DOWN;



type
  TMainForm = class(TForm)
    Panel1: TPanel;
    PaintBox1: TPaintBox;
    ActionList1: TActionList;
    Panel2: TPanel;
    Button1: TButton;
    RadioButton1: TRadioButton;
    Start: TAction;
    Timer1: TTimer;
    Button2: TButton;
    Stop: TAction;
    SpinEdit1: TSpinEdit;
    Label1: TLabel;
    Label2: TLabel;
    SpinEdit2: TSpinEdit;
    Image1: TImage;
    procedure PaintBox1Paint(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure StartExecute(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure StopExecute(Sender: TObject);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
    Bm : TBitmap; // Bitmapa po kojoj vrsimo iscrtavanje
    LopticaX, LopticaY : integer; // Trenutne koordinate loptice
    LopticaPomeraj : integer; // Pomeraj loptice u jedinici vremena koliko pomeraja na 20 milisekundi
    SmerKretanja : integer; // Pravac kretanja loptice
    LastTimeTick : longint; // Zadnje izmereno vreme
    Iscrtavanje : integer;

    // Dimenzija loptice
    VELICINA_LOPTE_X, VELICINA_LOPTE_Y : integer;

    procedure DrawOnBitmap;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

procedure TMainForm.PaintBox1Paint(Sender: TObject);
begin
     // Radimo kopiranje iz bitmape na paintbox
     // tako sto kopiramo samo deo koji nam treba
     BitBlt(PaintBox1.Canvas.Handle,
            PaintBox1.Canvas.ClipRect.Left,
            PaintBox1.Canvas.ClipRect.Top,
            PaintBox1.Canvas.ClipRect.Right-PaintBox1.Canvas.ClipRect.Left,
            PaintBox1.Canvas.ClipRect.Bottom-PaintBox1.Canvas.ClipRect.Top,
            Bm.Canvas.Handle,
            PaintBox1.Canvas.ClipRect.Left,
            PaintBox1.Canvas.ClipRect.Top,
            SRCCOPY
            )
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
     // Kreiramo bitmapu i podesavamo velicinu
     Bm := TBitmap.Create;
     Bm.Width := PaintBox1.Width;
     Bm.Height := PaintBox1.Height;
     Bm.PixelFormat := pf32bit;

     // Postavljamo pocetne koordinate loptice
     // kao i pomeraj
     LopticaX := PaintBox1.Width div 2;
     LopticaY := PaintBox1.Height div 2;
     LopticaPomeraj := 1;
     SmerKretanja := PRAVAC_DESNO;
     VELICINA_LOPTE_X := Image1.Width;
     VELICINA_LOPTE_Y := Image1.Height;

     // Pocetno iscrtavanje
     DrawOnBitmap;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
     // Oslobadjamo memoriju...
     Bm.Free;
end;

procedure TMainForm.DrawOnBitmap;
begin
     With Bm, Canvas do begin
     // Brisemo sliku
     Brush.Color := clWhite;
     Pen.Color := clBlack;
     FillRect(Rect(0,0,Width,Height));

     // Iscrtavamo lopticu
     BitBlt(Handle,
            LopticaX, LopticaY,
            VELICINA_LOPTE_X, VELICINA_LOPTE_Y,
            Image1.Canvas.Handle,
            0, 0,
            SRCCOPY)
     end; // With
end;

procedure TMainForm.StartExecute(Sender: TObject);
begin
     // Startujemo igricu...
     if RadioButton1.Checked then begin
     LopticaPomeraj := SpinEdit1.Value;
     LastTimeTick := GetTickCount;
     Timer1.Interval := SpinEdit2.Value;
     Timer1.Enabled := true;
     Exit;
     end; // if RadioButton1

     // A ako je izabran...
end;

procedure TMainForm.Timer1Timer(Sender: TObject);
Var Razlika : longint;
    Pomeri : integer;
begin
     Razlika := GetTickCount - LastTimeTick; // Koliko je proslo milisek. od zadnjeg pomeranja
     LastTimeTick := GetTickCount; // Pamtimo novo vreme
     Pomeri := Trunc((Razlika / 20) * LopticaPomeraj); // Koliko pomeramo
     if Pomeri < 1 then Exit; // Ako pomeramo manje od pixela

     Case SmerKretanja of
     PRAVAC_LEVO : begin
                   LopticaX := LopticaX - Pomeri;
                   if LopticaX < -VELICINA_LOPTE_X then LopticaX := Bm.Width;
                   end;
     PRAVAC_DESNO : begin
                    LopticaX := LopticaX + Pomeri;
                    if LopticaX > Bm.Width then LopticaX := 0;
                    end;
     PRAVAC_GORE : begin
                   LopticaY := LopticaY - Pomeri;
                   if LopticaY < -VELICINA_LOPTE_Y then LopticaY := Bm.Height;
                   end;
     PRAVAC_DOLE : begin
                   LopticaY := LopticaY + Pomeri;
                   if LopticaY > Bm.Height then LopticaY := 0;
                   end;
     end;

     DrawOnBitmap;
     PaintBox1.Repaint;
end;

procedure TMainForm.StopExecute(Sender: TObject);
begin
     // Prekidamo igricu...
     if RadioButton1.Checked then begin
     Timer1.Enabled := false;
     Exit;
     end; // if RadioButton1

     // A ako je izabran...
end;

procedure TMainForm.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
     // Hvatamo pravac pomeranja sa tastature...
     if Key in [VK_LEFT,VK_RIGHT,VK_UP,VK_DOWN] then SmerKretanja := Key;
end;

procedure TMainForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
    // Ovde maskiramo tastere da ostale kontrole ne bi reagovale 
    if Key in [VK_LEFT,VK_RIGHT,VK_UP,VK_DOWN] then Key := 0;
end;

end.
